function mondriaanplot(A, NumProcessors, Imbalance, Permutation, Iterations)
% mondriaanplot   Creates a coloured plot of the Mondriaan partitioning of
%                 a given matrix A over NumProcessors processors with a
%         maximum imbalance specified by Imbalance.
%         If desired, a permutation may also be specified in the
%         variable Permutation: 0 = no permutation,
%                               1 = move cut rows/columns to the front
%                               2 = move cut rows/columns to the middle
%                               3 = move cut rows/columns to the back.
%         It is also possible to set the desired maximum  number of
%         iterations in the variable Iterations to make animations.
%         Setting this value  < 0 will lead to a full splitting.
%         
%         Return value:
%             Nothing is returned, but a picture of the partitioning process is generated.
%
%         Usage:
%             mondriaanplot(A, NumProcessors, Imbalance, Permutation, Iterations);
%
if (nargin < 4)
    Permutation = 0;
    Iterations = -1;
end
if (nargin < 5)
    Iterations = -1;
end

Colours = colormap(hsv(NumProcessors + 1));

[B, s] = MatlabMondriaan(A, NumProcessors, Imbalance, Iterations, Permutation);

% Create figure.
clf
hold off

for i=1:NumProcessors
    C = (B == i);
    
    % The following code is based on spy.m, included with Matlab.
    %spy(C, Colours(i,:));
    [k, l] = find(C);
    plot(l, k, 'marker', '.', 'markersize', 2, 'linestyle', 'none', 'color', Colours(i,:));
    hold on
end

%Print some basic statistics.
[m, n] = size(A);
xlabel(['m = ', int2str(m), ', n = ', int2str(n), ', nz = ', int2str(nnz(A)), ', p = ', int2str(NumProcessors), ', eps = ', num2str(s(2)), ', max. com. = ', num2str(s(3)), ', com. vol. = ', num2str(s(4))]);

set(gca, 'xlim', [0 n+1], 'ylim', [0 m+1], 'grid', 'none', 'plotboxaspectratio', [n+1 m+1 1], 'ydir', 'reverse');

end
